@@ -3,15 +3,15 @@ cclib (http://cclib.sf.net) is (c) 2006, the cclib development team
 and licensed under the LGPL (http://www.gnu.org/copyleft/lgpl.html).
 """
 
-__revision__ = "$Revision: 834 $"
+__revision__ = "$Revision: 873 $"
 
 
 import re
 
 import numpy
 
-import utils
 import logfileparser
+import utils
 
 
 class GAMESS(logfileparser.Logfile):
@@ -55,10 +55,14 @@ class GAMESS(logfileparser.Logfile):
         self.firststdorient = True # Used to decide whether to wipe the atomcoords clean
         self.geooptfinished = False # Used to avoid extracting the final geometry twice
         self.cihamtyp = "none" # Type of CI Hamiltonian: saps or dets.
+        self.scftype = "none" # Type of SCF calculation: BLYP, RHF, ROHF, etc.
     
     def extract(self, inputfile, line):
         """Extract information from the file object inputfile."""
-        
+
+        if line [1:12] == "INPUT CARD>":
+            return
+
         # We are looking for this line:
         #           PARAMETERS CONTROLLING GEOMETRY SEARCH ARE
         #           ...
@@ -278,6 +282,13 @@ class GAMESS(logfileparser.Logfile):
                 parts = line.split()
                 rms = float(parts[3])
 
+
+            # FMO also prints two final one- and two-body gradients (see exam37):
+            #   (1) MAXIMUM GRADIENT =  0.0531540    RMS GRADIENT = 0.0189223
+            if len(parts) == 9:
+                maximum = float(parts[4])
+                rms = float(parts[8])
+
             self.geovalues.append([maximum, rms])
 
         if line[11:50] == "ATOMIC                      COORDINATES":
@@ -319,52 +330,90 @@ class GAMESS(logfileparser.Logfile):
 
             atomcoords = []
             line = inputfile.next()                
-            while line.strip():
+
+            for i in range(self.natom):
                 temp = line.strip().split()
                 atomcoords.append(map(float, temp[2:5]))
                 line = inputfile.next()
             self.atomcoords.append(atomcoords)
         
-        # This is the section with the SCF information.
-        if line.rstrip()[-15:] == "SCF CALCULATION":
+        # Section with SCF information.
+        #
+        # The space at the start of the search string is to differentiate from MCSCF.
+        # Everything before the search string is stored as the type of SCF.
+        # SCF types may include: BLYP, RHF, ROHF, UHF, etc.
+        #
+        # For example, in exam17 the section looks like this (note that this is GVB):
+        #          ------------------------
+        #          ROHF-GVB SCF CALCULATION
+        #          ------------------------
+        # GVB STEP WILL USE    119875 WORDS OF MEMORY.
+        #
+        #     MAXIT=  30   NPUNCH= 2   SQCDF TOL=1.0000E-05
+        #     NUCLEAR ENERGY=        6.1597411978
+        #     EXTRAP=T   DAMP=F   SHIFT=F   RSTRCT=F   DIIS=F  SOSCF=F
+        #
+        # ITER EX     TOTAL ENERGY       E CHANGE        SQCDF       DIIS ERROR
+        #   0  0      -38.298939963   -38.298939963   0.131784454   0.000000000
+        #   1  1      -38.332044339    -0.033104376   0.026019716   0.000000000
+        # ... and will be terminated by a blank line.
+        if line.rstrip()[-16:] == " SCF CALCULATION":
 
-            line = inputfile.next()
+            # Remember the type of SCF.
+            self.scftype = line.strip()[:-16]
 
-            while not "ITER EX" in line:
+            dashes = inputfile.next()
 
-                # Deal with the various versions:
+            while line [:5] != " ITER":
+
+                # GVB uses SQCDF for checking convergence (for example in exam17).
+                if "GVB" in self.scftype and "SQCDF TOL=" in line:
+                    scftarget = float(line.split("=")[-1])
+
+                # Normally however the density is used as the convergence criterium.
+                # Deal with various versions:
                 #   (GAMESS VERSION = 12 DEC 2003)
                 #     DENSITY MATRIX CONV=  2.00E-05  DFT GRID SWITCH THRESHOLD=  3.00E-04
                 #   (GAMESS VERSION = 22 FEB 2006)
                 #     DENSITY MATRIX CONV=  1.00E-05
                 #   (PC GAMESS version 6.2, Not DFT?)
                 #     DENSITY CONV=  1.00E-05
-                if "DENSITY CONV" in line or "DENSITY MATRIX CONV" in line:
-
+                elif "DENSITY CONV" in line or "DENSITY MATRIX CONV" in line:
                     scftarget = float(line.split()[-1])
 
                 line = inputfile.next()
 
             if not hasattr(self, "scftargets"):
                 self.scftargets = []
+
             self.scftargets.append([scftarget])
 
             if not hasattr(self,"scfvalues"):
                 self.scfvalues = []
+
             line = inputfile.next()
+
+            # Normally the iteration print in 6 columns.
+            # For ROHF, however, it is 5 columns, thus this extra parameter.
+            if "ROHF" in self.scftype:
+                valcol = 4
+            else:
+                valcol = 5
+
+            # SCF iterations are terminated by a blank line.
+            # The first four characters usually contains the step number.
+            # However, lines can also contain messages, including:
+            #   * * *   INITIATING DIIS PROCEDURE   * * *
+            #   CONVERGED TO SWOFF, SO DFT CALCULATION IS NOW SWITCHED ON
+            #   DFT CODE IS SWITCHING BACK TO THE FINER GRID
             values = []
             while line.strip():
-            # The SCF information is terminated by a blank line                    
                 try:
                     temp = int(line[0:4])
                 except ValueError:
-            # Occurs for:
-            #  * * *   INITIATING DIIS PROCEDURE   * * *
-            #  CONVERGED TO SWOFF, SO DFT CALCULATION IS NOW SWITCHED ON
-            #  DFT CODE IS SWITCHING BACK TO THE FINER GRID
                     pass
                 else:
-                    values.append([float(line.split()[5])])
+                    values.append([float(line.split()[valcol])])
                 line = inputfile.next()
             self.scfvalues.append(values)
 
@@ -612,11 +661,16 @@ class GAMESS(logfileparser.Logfile):
                 
                 numbers = inputfile.next() # Eigenvector numbers.
 
-                # Eigenvalues for these orbitals (in hartrees).
-                # Sometimes there are some blank lines before it.
+                # Sometimes there are some blank lines here.
                 while not line.strip():
                     line = inputfile.next()
-                self.moenergies[0].extend([utils.convertor(float(x), "hartree", "eV") for x in line.split()])
+
+                # Eigenvalues for these orbitals (in hartrees).
+                try:
+                    self.moenergies[0].extend([utils.convertor(float(x), "hartree", "eV") for x in line.split()])
+                except:
+                    self.logger.warning('MO section found but could not be parsed!')
+                    break;
 
                 # Orbital symmetries.
                 line = inputfile.next()
@@ -630,6 +684,10 @@ class GAMESS(logfileparser.Logfile):
                 oldatom ='0'
                 for i in range(self.nbasis):
                     line = inputfile.next()
+
+                    # If line is empty, break (ex. for FMO in exam37).
+                    if not line.strip(): break
+
                     # Fill atombasis and aonames only first time around
                     if readatombasis and base == 0:
                         aonames = []
@@ -726,24 +784,41 @@ class GAMESS(logfileparser.Logfile):
                         self.nocoeffs[base+j, i] = float(coeffs[j * 11:(j + 1) * 11])
                         j += 1
 
-        if line.find("NUMBER OF OCCUPIED ORBITALS") >= 0:
+        # We cannot trust this self.homos until we come to the phrase:
+        #   SYMMETRIES FOR INITAL GUESS ORBITALS FOLLOW
+        # which either is followed by "ALPHA" or "BOTH" at which point we can say
+        # for certain that it is an un/restricted calculations.
+        # Note that MCSCF calcs also print this search string, so make sure
+        #   that self.homos does not exist yet.
+        if line[1:28] == "NUMBER OF OCCUPIED ORBITALS" and not hasattr(self,'homos'):
             homos = [int(line.split()[-1])-1]
             line = inputfile.next()
             homos.append(int(line.split()[-1])-1)
-            # Note that we cannot trust this self.homos until we come to
-            # a line that contains the phrase:
-            # "SYMMETRIES FOR INITAL GUESS ORBITALS FOLLOW"
-            # which either is followed by "ALPHA" or "BOTH"
-            # at which point we can say for certain that it is an
-            # un/restricted calculations
             self.homos = numpy.array(homos, "i")
 
+        
         if line.find("SYMMETRIES FOR INITIAL GUESS ORBITALS FOLLOW") >= 0:
-            # Not unrestricted, so lop off the second index
+            # Not unrestricted, so lop off the second index.
+            # In case the search string above was not used (ex. FMO in exam38),
+            #   we can try to use the next line which should also contain the
+            #   number of occupied orbitals.
             if line.find("BOTH SET(S)") >= 0:
+                nextline = inputfile.next()
+                if "ORBITALS ARE OCCUPIED" in nextline:
+                    homos = int(nextline.split()[0])-1
+                    if hasattr(self,"homos"):
+                        try:
+                            assert self.homos[0] == homos
+                        except AssertionError:
+                            self.logger.warning("Number of occupied orbitals not consistent. This is normal for ECP and FMO jobs.")
+                    else:
+                        self.homos = [homos]
                 self.homos = numpy.resize(self.homos, [1])
 
-        if line.find("TOTAL NUMBER OF ATOMS") == 1:
+        # Set the total number of atoms, only once.
+        # Normally GAMESS print TOTAL NUMBER OF ATOMS, however in some cases
+        #   this is slightly different (ex. lower case for FMO in exam37).
+        if not hasattr(self,"natom") and "NUMBER OF ATOMS" in line.upper():
             self.natom = int(line.split()[-1])
             
         if line.find("NUMBER OF CARTESIAN GAUSSIAN BASIS") == 1 or line.find("TOTAL NUMBER OF BASIS FUNCTIONS") == 1:
