@@ -3,14 +3,15 @@ cclib (http://cclib.sf.net) is (c) 2006, the cclib development team
 and licensed under the LGPL (http://www.gnu.org/copyleft/lgpl.html).
 """
 
-__revision__ = "$Revision: 844 $"
+__revision__ = "$Revision: 878 $"
 
 
 import re
+
 import numpy
 
-import utils
 import logfileparser
+import utils
 
 
 class Gaussian(logfileparser.Logfile):
@@ -58,13 +59,29 @@ class Gaussian(logfileparser.Logfile):
 
         # Used to index self.scftargets[].
         SCFRMS, SCFMAX, SCFENERGY = range(3)
+
         # Flag that indicates whether it has reached the end of a geoopt.
         self.optfinished = False
+
         # Flag for identifying Coupled Cluster runs.
         self.coupledcluster = False
+
         # Fragment number for counterpoise calculations (normally zero).
         self.counterpoise = 0
 
+        # Flag for identifying ONIOM calculations.
+        self.oniom = False
+
+    def after_parsing(self):
+
+        # Correct the percent values in the etsecs in the case of
+        # a restricted calculation. The following has the
+        # effect of including each transition twice.
+        if hasattr(self, "etsecs") and len(self.homos) == 1:
+            new_etsecs = [[(x[0], x[1], x[2] * numpy.sqrt(2)) for x in etsec]
+                          for etsec in self.etsecs]
+            self.etsecs = new_etsecs
+            
     def extract(self, inputfile, line):
         """Extract information from the file object inputfile."""
         
@@ -83,15 +100,16 @@ class Gaussian(logfileparser.Logfile):
         
         # Extract the atomic numbers and coordinates from the input orientation,
         #   in the event the standard orientation isn't available.
-        if line.find("Input orientation") > -1 or line.find("Z-Matrix orientation") > -1:
+        if not self.optfinished and line.find("Input orientation") > -1 or line.find("Z-Matrix orientation") > -1:
 
             # If this is a counterpoise calculation, this output means that
             #   the supermolecule is now being considered, so we can set:
             self.counterpoise = 0
 
             self.updateprogress(inputfile, "Attributes", self.cupdate)
-                    
-            self.inputcoords = []
+            
+            if not hasattr(self, "inputcoords"):
+                self.inputcoords = []
             self.inputatoms = []
             
             hyphens = inputfile.next()
@@ -114,7 +132,7 @@ class Gaussian(logfileparser.Logfile):
                 self.natom = len(self.atomnos)
 
         # Extract the atomic numbers and coordinates of the atoms.
-        if not self.optfinished and line[25:45] == "Standard orientation":
+        if not self.optfinished and line.strip() == "Standard orientation:":
 
             self.updateprogress(inputfile, "Attributes", self.cupdate)
 
@@ -136,7 +154,7 @@ class Gaussian(logfileparser.Logfile):
             while line != hyphens:
                 broken = line.split()
                 atomnos.append(int(broken[1]))
-                atomcoords.append(map(float, broken[3:6]))
+                atomcoords.append(map(float, broken[-3:]))
                 line = inputfile.next()
             self.atomcoords.append(atomcoords)
             if not hasattr(self, "natom"):
@@ -218,8 +236,9 @@ class Gaussian(logfileparser.Logfile):
                         self.progress.update(step, "AM1 Convergence")
                         oldstep = step
                         
-                parts = line.strip().split()
-                self.scfvalues[0].append(self.float(parts[-1][:-1]))
+                if line[1:4] == "It=":
+                    parts = line.strip().split()
+                    self.scfvalues[0].append(self.float(parts[-1][:-1]))
                 line = inputfile.next()
 
         # Note: this needs to follow the section where 'SCF Done' is used
@@ -319,6 +338,42 @@ class Gaussian(logfileparser.Logfile):
 
             self.geovalues.append(newlist)
 
+        # Gradients.
+        # Read in the cartesian energy gradients (forces) from a block like this:
+        # -------------------------------------------------------------------
+        # Center     Atomic                   Forces (Hartrees/Bohr)
+        # Number     Number              X              Y              Z
+        # -------------------------------------------------------------------
+        # 1          1          -0.012534744   -0.021754635   -0.008346094
+        # 2          6           0.018984731    0.032948887   -0.038003451
+        # 3          1          -0.002133484   -0.006226040    0.023174772
+        # 4          1          -0.004316502   -0.004968213    0.023174772
+        # ------------------------------------------------------------------
+        #
+        # Then optimization is done in internal coordinates, Gaussian also
+        # print the forces in internal coordinates, which can be produced from 
+        # the above. This block looks like this:
+        # Variable       Old X    -DE/DX   Delta X   Delta X   Delta X     New X
+        #                                 (Linear)    (Quad)   (Total)
+        #   ch        2.05980   0.01260   0.00000   0.01134   0.01134   2.07114
+        #   hch        1.75406   0.09547   0.00000   0.24861   0.24861   2.00267
+        #   hchh       2.09614   0.01261   0.00000   0.16875   0.16875   2.26489
+        #         Item               Value     Threshold  Converged?
+        if line[37:43] == "Forces":
+
+            if not hasattr(self, "grads"):
+                self.grads = []
+
+            header = inputfile.next()
+            dashes = inputfile.next()
+            line = inputfile.next()
+            forces = []
+            while line != dashes:
+                n,nuclear,Fx,Fy,Fz = line.split()
+                forces.append([float(Fx),float(Fy),float(Fz)])
+                line = inputfile.next()
+            self.grads.append(forces)                
+
         # Charge and multiplicity.
         # If counterpoise correction is used, multiple lines match.
         # The first one contains charge/multiplicity of the whole molecule.:
@@ -378,6 +433,9 @@ class Gaussian(logfileparser.Logfile):
             # For counterpoise fragments, skip these lines.
             if self.counterpoise != 0: return
 
+            # For ONIOM calcs, ignore this section in order to bypass assertion failure.
+            if self.oniom: return
+
             self.updateprogress(inputfile, "Eigenvalues", self.fupdate)
             self.moenergies = [[]]
             HOMO = -2
@@ -444,7 +502,7 @@ class Gaussian(logfileparser.Logfile):
         #      0.2941249355D+01 -0.9996722919D-01  0.1559162750D+00
         #      0.6834830964D+00  0.3995128261D+00  0.6076837186D+00
         #      0.2222899159D+00  0.7001154689D+00  0.3919573931D+00
-        if line[1:14] == "AO basis set ":
+        if line[1:16] == "AO basis set in":
         
             # For counterpoise fragment calcualtions, skip these lines.
             if self.counterpoise != 0: return
@@ -583,10 +641,13 @@ class Gaussian(logfileparser.Logfile):
             # Excited State   1:   Singlet-BU     5.3351 eV  232.39 nm  f=0.1695
             # (unrestricted calc) (first excited state is 2!)
             # Excited State   2:   ?Spin  -A      0.1222 eV 10148.75 nm  f=0.0000
-            parts = line[36:].split()
-            self.etenergies.append(utils.convertor(self.float(parts[0]), "eV", "cm-1"))
-            self.etoscs.append(self.float(parts[4].split("=")[1]))
-            self.etsyms.append(line[21:36].strip())
+            # (Gaussian 09 ZINDO)
+            # Excited State   1:      Singlet-?Sym    2.5938 eV  478.01 nm  f=0.0000  <S**2>=0.000
+            p = re.compile(":(?P<sym>.*?)(?P<energy>-?\d*\.\d*) eV")
+            groups = p.search(line).groups()
+            self.etenergies.append(utils.convertor(self.float(groups[1]), "eV", "cm-1"))
+            self.etoscs.append(self.float(line.split("f=")[-1].split()[0]))
+            self.etsyms.append(groups[0].strip())
             
             line = inputfile.next()
 
@@ -612,25 +673,45 @@ class Gaussian(logfileparser.Logfile):
                 toMO = int(p.match(toMO).group())-1 # subtract 1 so that it is an index into moenergies
 
                 percent = self.float(t[1])
-                if len(self.homos)==1:
-                    # For restricted calculations, the following has the
-                    # effect of including each transition twice
-                    percent *= numpy.sqrt(2)
+                # For restricted calculations, the percentage will be corrected
+                # after parsing (see after_parsing() above).
                 CIScontrib.append([(fromMO, frommoindex), (toMO, tomoindex), percent])
                 line = inputfile.next()
             self.etsecs.append(CIScontrib)
 
-        # Circular dichroism data.
-        if line[1:52] == "<0|r|b> * <b|rxdel|0>  (Au), Rotatory Strengths (R)":
+# Circular dichroism data (different for G03 vs G09)
+
+# G03
+
+## <0|r|b> * <b|rxdel|0>  (Au), Rotatory Strengths (R) in
+## cgs (10**-40 erg-esu-cm/Gauss)
+##       state          X           Y           Z     R(length)
+##         1         0.0006      0.0096     -0.0082     -0.4568
+##         2         0.0251     -0.0025      0.0002     -5.3846
+##         3         0.0168      0.4204     -0.3707    -15.6580
+##         4         0.0721      0.9196     -0.9775     -3.3553
+
+# G09
+
+## 1/2[<0|r|b>*<b|rxdel|0> + (<0|rxdel|b>*<b|r|0>)*]
+## Rotatory Strengths (R) in cgs (10**-40 erg-esu-cm/Gauss)
+##       state          XX          YY          ZZ     R(length)     R(au)
+##         1        -0.3893     -6.7546      5.7736     -0.4568     -0.0010
+##         2       -17.7437      1.7335     -0.1435     -5.3845     -0.0114
+##         3       -11.8655   -297.2604    262.1519    -15.6580     -0.0332
+
+        if (line[1:52] == "<0|r|b> * <b|rxdel|0>  (Au), Rotatory Strengths (R)" or
+            line[1:50] == "1/2[<0|r|b>*<b|rxdel|0> + (<0|rxdel|b>*<b|r|0>)*]"):
 
             self.etrotats = []
-            inputfile.next()
-            inputfile.next()
+            inputfile.next() # Units
+            headers = inputfile.next() # Headers
+            Ncolms = len(headers.split())
             line = inputfile.next()
             parts = line.strip().split()
-            while len(parts) == 5:
+            while len(parts) == Ncolms:
                 try:
-                    R = self.float(parts[-1])
+                    R = self.float(parts[4])
                 except ValueError:
                     # nan or -nan if there is no first excited state
                     # (for unrestricted calculations)
@@ -654,6 +735,9 @@ class Gaussian(logfileparser.Logfile):
             # For counterpoise fragment, skip these lines.
             if self.counterpoise != 0: return
 
+            # For ONIOM calcs, ignore this section in order to bypass assertion failure.
+            if self.oniom: return
+
             # If nbasis was already parsed, check if it changed.
             nbasis = int(line.split('=')[1].split()[0])
             if hasattr(self, "nbasis"):
@@ -667,6 +751,9 @@ class Gaussian(logfileparser.Logfile):
             # For counterpoise fragment, skip these lines.
             if self.counterpoise != 0: return
 
+            # For ONIOM calcs, ignore this section in order to bypass assertion failure.
+            if self.oniom: return
+
             # If nmo was already parsed, check if it changed.
             nmo = int(line.split('=')[1].split()[0])
             if hasattr(self, "nmo"):
@@ -736,6 +823,11 @@ class Gaussian(logfileparser.Logfile):
                 self.updateprogress(inputfile, "Coefficients", self.fupdate)
                          
                 colmNames = inputfile.next()   
+
+                if not colmNames.split():
+                    self.logger.warning("Molecular coefficients header found but no coefficients.")
+                    break;
+
                 if base==0 and int(colmNames.split()[0])!=1:
                     # Implies that this is a POP=REGULAR calculation
                     # and so, only aonames (not mocoeffs) will be extracted
@@ -872,6 +964,10 @@ class Gaussian(logfileparser.Logfile):
             if line[42:50] == "fragment":
                 self.counterpoise = int(line[51:54])
 
+        # This will be printed only during ONIOM calcs; use it to set a flag
+        # that will allow assertion failures to be bypassed in the code.
+        if line[1:7] == "ONIOM:":
+            self.oniom = True
 
 if __name__ == "__main__":
     import doctest, gaussianparser
