@@ -1,7 +1,24 @@
+#
+# GaussSum (http://gausssum.sf.net)
+# Copyright (C) 2006-2009 Noel O'Boyle <baoilleach@gmail.com>
+#
+# This program is free software; you can redistribute and/or modify it
+# under the terms of the GNU General Public License as published by the
+# Free Software Foundation; either version 2, or (at your option) any later
+# version.
+#
+# This program is distributed in the hope that it will be useful, but
+# WITHOUT ANY WARRANTY, without even the implied warranty of
+# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+# General Public License for more details
+
 import os
 import sys
 import math
+import stat
 import numpy
+import glob
+
 from gausssum.utils import GaussianSpectrum, levelname, percent
 from gausssum.cclib.parser.utils import convertor
 
@@ -10,6 +27,159 @@ from gnupy import Gnuplot
 from plot import DisplayPlot
 from folder import folder
 
+def createEDDM(screen, logfile, contrib, gaussdir, unres):
+
+    def cubman(cmds):
+        if cmds[0] == "sc":
+            output.write("echo Scaling %s by %s\n" % (cmds[1], cmds[-1]))
+        elif cmds[0] == "a":
+            output.write("echo Adding to %s\n" % cmds[3])
+        elif cmds[0] == "sq":
+            output.write("echo Squaring %s\n" % cmds[1])
+        elif cmds[0] == "su":
+            output.write("echo Subtracting %s from %s\n" % (cmds[1], cmds[3]))        
+        lines = []
+        lines.append("echo %s > tmp.txt" % cmds[0])
+        for cmd in cmds[1:]:
+            lines.append("echo %s >> tmp.txt" % cmd)
+        cmdname = syscmd[CAT]
+        lines.append("%s tmp.txt | %s%scubman > tmp2.txt\n" % (cmdname, envvar('G03DIR'), os.sep))
+        output.write("\n".join(lines))
+
+    def envvar(x):
+        if windows:
+            return "%" + x + "%"
+        else:
+            return "${" + x + "}"
+
+    def createcubes(mo):       
+        cube = "mo%s.cub" % getmo(mo)
+        report = "echo Creating cubefile %s" % cube
+        orb = mo[0] + 1
+        if unres:
+            calctype = ['alpha', 'beta'][mo[1]]
+            orb += len(logfile.moenergies[0]) * mo[1]
+        cmd = "%s%scubegen 0 MO=%d %s %s -2 h\n" % (
+                    envvar('G03DIR'), os.sep, orb, fchkpoint, cube)
+        if windows:
+            line = "if not exist %s %s & %s\n" % (cube, report, cmd)
+        else:
+            line = 'if [ ! -e "%s" ]\nthen\n  %s\n  %s\nfi\n' % (cube, report, cmd)
+        output.write(line)
+
+    def getmo(mo):
+        suffix = ""
+        if unres:
+            suffix = ["A", "B"][mo[1]]
+        return "%d%s" % (mo[0] + 1, suffix)
+        
+    def createsquares(mo):
+        cube = "mo%s.cub" % getmo(mo)
+        sq = "sq%s.cub" % getmo(mo)
+        if windows:
+            output.write("if not exist %s (\n" % sq)
+        else:
+            output.write("if [ ! -e %s ]\nthen\n" % sq)
+        cubman(["sq", cube, "y", sq, "y"])
+        if windows:
+            output.write(")\n")
+        else:
+            output.write("fi\n")
+
+    def formchk(filename):
+        filename = filename.split(fakeossep)[-1]
+        fchkpoint = filename.replace('.chk', '.fch')
+        report = "echo Formatting %s to %s" % (filename, fchkpoint)
+        cmd = "%s%sformchk %s %s > tmp2.txt\n" % (envvar('G03DIR'), os.sep, filename,
+                                      fchkpoint)
+        if windows:
+            line = "if not exist %s %s & %s\n" % (fchkpoint, report, cmd)
+        else:
+            line = 'if [ ! -e "%s" ]\nthen\n  %s\n  %s\nfi\n' % (fchkpoint, report, cmd)
+        return fchkpoint, line
+
+    windows = sys.platform == "win32"
+##    windows = False
+    fakeossep = os.sep
+##    fakeossep = "\\"
+##    os.sep = "/"
+
+    COPY, DELETE, CAT, MOVE = range(4)
+    syscmd = ['cp', 'rm', 'cat', 'mv']
+    if windows:
+         syscmd = ['copy', 'del', 'type', 'move']
+
+    fchkpoint = glob.glob(os.path.join(gaussdir, "*.fch"))
+    line = ""
+    if len(fchkpoint) == 0:
+        chkpoint = glob.glob(os.path.join(gaussdir, "*.chk"))
+        fchkpoint, line = formchk(chkpoint[0])
+    else:
+        fchkpoint = fchkpoint[0].split(fakeossep)[-1]
+        # Error!
+
+    screen.write("Creating the EDDM script file for %s\n" % fchkpoint)
+
+    if windows:
+        output = open(os.path.join(gaussdir, "eddm.bat"), "w")        
+        output.write("@echo off\n")
+    else:
+        finalsection = []
+        output = open(os.path.join(gaussdir, "eddm.sh"), "w")
+        os.chmod(os.path.join(gaussdir, "eddm.sh"), stat.S_IRWXU) # Make executable
+    if line:
+        output.write(line)
+    output.write("echo === Using formatted checkpoint %s ===\n" % fchkpoint)
+    for i in range(len(logfile.etenergies)): # For each transition
+        if windows:
+            output.write("""if "%%1" == "%d" goto :tran%d\n""" % (i+1, i+1))
+        else:
+            finalsection.append("""if [ "$1" == "%d" ]\nthen\n  tran%d\nfi\n""" % (i+1, i+1))
+    line = "echo You need to specify a transition in the range 1 to %d\n" % len(logfile.etenergies)    
+    if windows:
+        output.write("%s\ngoto :end\n" % line)
+    else:
+        finalsection.append(line)
+
+    for i in range(len(logfile.etenergies)): # For each transition
+        if windows:
+            output.write(":tran%d\n" % (i+1))
+        else:
+            output.write("function tran%d\n{\n" % (i+1))
+            
+        totalcon = sum(contrib[i])
+        for sec in logfile.etsecs[i]:
+            createcubes(sec[0])
+            createcubes(sec[1])
+            createsquares(sec[0])
+            createsquares(sec[1])
+        scale = float(contrib[i][0]) / totalcon # Should square totalcon?
+        cubman(["sc","sq%s.cub" % getmo(logfile.etsecs[i][0][0]),"y","before.cub","y",str(scale)])
+        cubman(["sc","sq%s.cub" % getmo(logfile.etsecs[i][0][1]),"y","after.cub","y",str(scale)])
+        for j in range(1, len(logfile.etsecs[i])):
+            scale = float(contrib[i][j]) / totalcon # Should square totalcon?
+            cubman(["sc","sq%s.cub" % getmo(logfile.etsecs[i][j][0]),"y","tmp.cub","y",str(scale)])
+            cubman(["a","tmp.cub","y","before.cub","y","tmp2.cub","y"])
+            output.write("%s tmp2.cub before.cub\n" % syscmd[MOVE])
+            cubman(["sc","sq%s.cub" % getmo(logfile.etsecs[i][j][1]),"y","after.cub","y",str(scale)])
+            cubman(["a","tmp.cub","y","after.cub","y","tmp2.cub","y"])
+            output.write("%s tmp2.cub after.cub\n" % syscmd[MOVE])
+        output.write("%s tmp.cub\n" % syscmd[DELETE])
+
+        cubman(["su","before.cub","y","after.cub","y",
+                "trans%d.cub" % (i+1,),"y"])
+        # output.write("del before.cub\ndel after.cub\n")
+        if windows:
+            output.write("goto :end\n")
+        else:
+            output.write("}\n")
+
+    if windows:
+        output.write(":end\n")
+    else:
+        output.write("\n".join(finalsection))
+    output.close()
+            
 def readorbital_data(inputfile):
     # Reads in all data from orbital_data.txt
 
@@ -54,7 +224,9 @@ def readorbital_data(inputfile):
     return HOMO,NBasis,NGroups,groupname,groupatoms,contrib,evalues
 
 
-def ET(root,screen,logfile,logfilename,start,end,numpts,FWHM,UVplot,gnuplotexec): # inputfilename is the name of a TD-DFT output file
+def ET(root,screen,logfile,logfilename,
+       start,end,numpts,FWHM,UVplot
+       ,gnuplotexec, EDDM): 
 
     screen.write("Starting to analyse the electronic transitions\n")
 
@@ -94,6 +266,7 @@ def ET(root,screen,logfile,logfilename,start,end,numpts,FWHM,UVplot,gnuplotexec)
                     # charge density [before, after] for each of the groups for each of the transitions
 
         majorCIS, minorCIS = [[] for x in logfile.etenergies], [[] for x in logfile.etenergies]
+        allpercent = [[] for x in logfile.etenergies]
         for i in range(len(logfile.etenergies)): # For each transition
             if orbdata:
                 CD=[ [0, 0] for x in range(NGroups)]
@@ -101,17 +274,17 @@ def ET(root,screen,logfile,logfilename,start,end,numpts,FWHM,UVplot,gnuplotexec)
             for j in range(len(CIS[i])): # For each contribution
                 thisCIS = CIS[i][j]
                 mycontrib = thisCIS[2] ** 2
-                if not unres:
-                    mycontrib *= 2
                 totcontribs += mycontrib # tot up the contribs
                 percontrib = int(mycontrib * 100 + .5)
                 if orbdata:
-                    for k in range(NGroups):
+                    for k in range(NGroups):    
                         CD[k][0] = (CD[k][0] + mycontrib *
                                     contrib[0, thisCIS[0][0], k])
                         CD[k][1] = (CD[k][1] + mycontrib *
-                                    contrib[0, thisCIS[i][j][1][0], k])
-                sign = ["", "-"][thisCIS[2] < 0]
+                                    contrib[0, thisCIS[1][0], k])
+                sign = ""
+                if thisCIS[2] < 0:
+                    sign = "-"
                 alphabeta = ["", ""]
                 if unres:
                     alphabeta = [["(A)", "(B)"][thisCIS[0][1]], ["(A)", "(B)"][thisCIS[1][1]]]
@@ -122,6 +295,7 @@ def ET(root,screen,logfile,logfilename,start,end,numpts,FWHM,UVplot,gnuplotexec)
                     majorCIS[i].append(CIStext)
                 elif percontrib >= 2: #Minor contributions (>=2%)
                     minorCIS[i].append(CIStext)
+                allpercent[i].append(percontrib)
                     
             if orbdata:
                 for j in range(len(groupname)): # The charge densities are scaled so that they add to one
@@ -129,11 +303,12 @@ def ET(root,screen,logfile,logfilename,start,end,numpts,FWHM,UVplot,gnuplotexec)
                     CD[j][1]=CD[j][1]/totcontribs
                     screenCD[i]=screenCD[i]+percent(CD[j][0])+"-->"+percent(CD[j][1])+" ("+percent(round(CD[j][1],2)-round(CD[j][0],2))+")\t"
                                                                                     
-            
+        if EDDM:
+            createEDDM(screen, logfile, allpercent, gaussdir, unres)
 
     # Write UVData.txt containing info on the transitions
 
-        fileoutput=""
+        fileoutput = ""
         for i in range(len(logfile.etenergies)): # For each transition
             temp = [i+1,logfile.etenergies[i],
                     convertor(logfile.etenergies[i],"cm-1","nm"),
@@ -172,26 +347,27 @@ def ET(root,screen,logfile,logfilename,start,end,numpts,FWHM,UVplot,gnuplotexec)
             outputfile.write("\n")
         outputfile.close()
 
-        # Plot the UV Spectrum using Gnuplot
+        if root:
+            # Plot the UV Spectrum using Gnuplot
 
-        screen.write("Plotting using Gnuplot\n")
+            screen.write("Plotting using Gnuplot\n")
 
-        if max(t.spectrum[0,:])<1E-8: # Gnuplot won't draw it if the spectrum is flat
-            screen.write("There are no peaks in this wavelength range!\n")
-        else:
-            g = Gnuplot(gnuplotexec)
-            g.commands("set ytics nomirror",
-                       "set y2tics",
-                       "set y2label 'Oscillator strength'",
-                       "set xlabel 'Wavelength (nm)'",
-                       "set ylabel 'epsilon'",
-                       "set xrange [%d:%d]" % (start,end) )
-            xvalues_nm = [convertor(x,"cm-1","nm") for x in t.xvalues]
-            g.data(zip(xvalues_nm,t.spectrum[0,:]),"notitle with lines")
-            energies_nm = [convertor(x,"cm-1","nm") for x in logfile.etenergies]
-            g.data(zip(energies_nm,logfile.etoscs),"axes x1y2 notitle with impulses")
+            if max(t.spectrum[0,:])<1E-8: # Gnuplot won't draw it if the spectrum is flat
+                screen.write("There are no peaks in this wavelength range!\n")
+            else:
+                g = Gnuplot(gnuplotexec)
+                g.commands("set ytics nomirror",
+                           "set y2tics",
+                           "set y2label 'Oscillator strength'",
+                           "set xlabel 'Wavelength (nm)'",
+                           "set ylabel 'epsilon'",
+                           "set xrange [%d:%d]" % (start,end) )
+                xvalues_nm = [convertor(x,"cm-1","nm") for x in t.xvalues]
+                g.data(zip(xvalues_nm,t.spectrum[0,:]),"notitle with lines")
+                energies_nm = [convertor(x,"cm-1","nm") for x in logfile.etenergies]
+                g.data(zip(energies_nm,logfile.etoscs),"axes x1y2 notitle with impulses")
 
-            DisplayPlot(root,g,"UV-Vis Spectrum")
+                DisplayPlot(root,g,"UV-Vis Spectrum")
 
     else:
 #######################################################
@@ -250,30 +426,31 @@ def ET(root,screen,logfile,logfilename,start,end,numpts,FWHM,UVplot,gnuplotexec)
             outputfile.write("\n")
         outputfile.close()
 
-        # Plot the UV Spectrum using Gnuplot
+        if root:
+            # Plot the UV Spectrum using Gnuplot
 
-        screen.write("Plotting using Gnuplot\n")
+            screen.write("Plotting using Gnuplot\n")
 
-        if max(t.spectrum[0,:])<1E-8: # Gnuplot won't draw it if the spectrum is flat
-            screen.write("There are no peaks in this wavelength range!\n")
-        else:
-            g = Gnuplot(gnuplotexec)
-            g.commands("set ytics nomirror",
-                       "set y2tics",
-                       "set y2label 'R (length) / 1e-40'",
-                       "set xlabel 'Wavelength (nm)'",
-                       "set ylabel 'epsilon'",
-                       "set xrange [%d:%d]" % (start,end) )
-            xvalues_nm = [convertor(x,"cm-1","nm") for x in t.xvalues]
-            g.data(zip(xvalues_nm,t.spectrum[0,:]),"notitle with lines")
-            energies_nm = [convertor(x,"cm-1","nm") for x in logfile.etenergies]
-            g.data(zip(energies_nm,logfile.etrotats),"axes x1y2 notitle with impulses")
+            if max(t.spectrum[0,:])<1E-8: # Gnuplot won't draw it if the spectrum is flat
+                screen.write("There are no peaks in this wavelength range!\n")
+            else:
+                g = Gnuplot(gnuplotexec)
+                g.commands("set ytics nomirror",
+                           "set y2tics",
+                           "set y2label 'R (length) / 1e-40'",
+                           "set xlabel 'Wavelength (nm)'",
+                           "set ylabel 'epsilon'",
+                           "set xrange [%d:%d]" % (start,end) )
+                xvalues_nm = [convertor(x,"cm-1","nm") for x in t.xvalues]
+                g.data(zip(xvalues_nm,t.spectrum[0,:]),"notitle with lines")
+                energies_nm = [convertor(x,"cm-1","nm") for x in logfile.etenergies]
+                g.data(zip(energies_nm,logfile.etrotats),"axes x1y2 notitle with impulses")
 
-            # line="set ytics nomirror\nset y2tics\nset y2label 'R (length) / 1e-40'\n"
-            # line=line+"set xlabel 'Wavelength (nm)'\nset ylabel 'epsilon'\nset xrange ["+str(start)+":"+str(end)+"]\n"
-            # line=line+"plot '"+os.path.join(gaussdir,"CDSpectrum.txt")+"' using 1:3 notitle with lines, '"+os.path.join(gaussdir,"CDSpectrum.txt")+"' using 4:6 axes x1y2 notitle with impulses\n"
+                # line="set ytics nomirror\nset y2tics\nset y2label 'R (length) / 1e-40'\n"
+                # line=line+"set xlabel 'Wavelength (nm)'\nset ylabel 'epsilon'\nset xrange ["+str(start)+":"+str(end)+"]\n"
+                # line=line+"plot '"+os.path.join(gaussdir,"CDSpectrum.txt")+"' using 1:3 notitle with lines, '"+os.path.join(gaussdir,"CDSpectrum.txt")+"' using 4:6 axes x1y2 notitle with impulses\n"
 
-            DisplayPlot(root,g,"Circular dichroism spectrum")
+                DisplayPlot(root,g,"Circular dichroism spectrum")
 
     screen.write("Finished")
 
