@@ -1,6 +1,6 @@
-# -*- coding: cp1252 -*-
 #
-# Copyright (C) 2006 Noel O'Boyle <oboylen2@mail.dcu.ie>
+# GaussSum (http://gausssum.sf.net)
+# Copyright (C) 2006-2009 Noel O'Boyle <baoilleach@gmail.com>
 #
 # This program is free software; you can redistribute and/or modify it
 # under the terms of the GNU General Public License as published by the
@@ -21,6 +21,7 @@ import traceback
 import copy             # For deepcopy...until I find a better way of doing this
 import ConfigParser     # For writing the settings to an .ini file
 import logging
+import glob
 
 from gausssum.cclib.parser import ADF, GAMESS, Gaussian, ccopen
 from gausssum.preferencesbox import PreferencesPopupBox
@@ -32,6 +33,7 @@ from gausssum.search import Search
 from gausssum.vibfreq import Vibfreq
 from gausssum.scf import SCF
 from gausssum.utils import *
+from gausssum.folder import folder
 
 import os, sys
 if hasattr(sys, "frozen"): # i.e. if using py2exe
@@ -55,7 +57,10 @@ class App:    # This sets up the GUI
         
         self.scrollbar=Scrollbar(frame2)
         self.scrollbar.pack(side=RIGHT,fill=Y)
-        self.txt=Text(frame2,yscrollcommand=self.scrollbar.set,width=90,height=21)
+        if sys.platform == "win32": 
+            self.txt=Text(frame2,font=("Courier New",8),yscrollcommand=self.scrollbar.set,width=90,height=21)
+        else: # Avoid font problems (? I don't remember the reason for this)
+            self.txt=Text(frame2,yscrollcommand=self.scrollbar.set,width=90,height=21)
         self.txt.pack(expand=1,fill=Y)
         self.scrollbar.config(command=self.txt.yview)
 
@@ -271,6 +276,14 @@ class App:    # This sets up the GUI
             self.FWHM.grid(row=0,column=7)
             self.FWHM.insert(0,self.settings['uvvis.fwhm'])
 
+            self.eddm = IntVar()
+            self.eddmbtn = Checkbutton(frame7, text="Create EDDM script?", variable=self.eddm)
+            self.eddmbtn.pack(anchor=W)
+            gaussdir = folder(self.screen, self.logfile.filename, create=False)
+            if (len(glob.glob(os.path.join(gaussdir, "*.fch"))) == 0 and
+                len(glob.glob(os.path.join(gaussdir, "*.chk"))) == 0):
+                self.eddmbtn.configure(state=DISABLED)
+
 
 ##	elif s=="EDDM":
 ##            Label(self.frame4,text="Transitions:").pack(side=LEFT)
@@ -389,7 +402,7 @@ class App:    # This sets up the GUI
             elif s=="MO":
                 worked = Popanalysis(self.root,self.screen,self.data,self.logfile.filename,float(self.start.get()),float(self.end.get()),self.MOplot.get(),float(self.FWHM.get()),self.makeorigin.get(),self.settings['global settings.gnuplot'])
             elif s=="UVVIS":
-                worked = ET(self.root,self.screen,self.data,self.logfile.filename,int(self.start.get()),int(self.end.get()),int(self.numpts.get()),float(self.FWHM.get()),self.UVplot.get(),self.settings['global settings.gnuplot'])
+                worked = ET(self.root,self.screen,self.data,self.logfile.filename,int(self.start.get()),int(self.end.get()),int(self.numpts.get()),float(self.FWHM.get()),self.UVplot.get(),self.settings['global settings.gnuplot'], self.eddm.get())
 ##            elif s=="EDDM":
 ##                worked=GaussSum.EDDM.EDDM(self.screen,self.logfile,self.trans.get(),self.settings['eddm.cubman'],self.settings['eddm.formchk'],self.settings['eddm.cubegen'])
 ##            elif s=="NMR":
@@ -531,63 +544,64 @@ class App:    # This sets up the GUI
         if self.settings!=oldsettings: # if there are any changes
             self.saveprefs()
 
+    def getsettingsfile(self):
+        # Check for GaussSum.ini in $APPDATA/GaussSum2.2 (XP)
+        #                        or $HOME/.GaussSum2.2 (Linux)
+        if sys.platform == "win32":
+            settingsfile = os.path.join(os.getenv("APPDATA"),"GaussSum2.2","GaussSum.ini")
+        else:
+            settingsfile = os.path.join(os.getenv("HOME"), ".GaussSum2.2","GaussSum.ini")
+        return settingsfile
+
     def saveprefs(self):
         # Save the settings
-        appdata=os.getenv("HOME")
-        settingsfile=os.path.join(appdata,"GaussSum2.1","GaussSum.ini")
+        settingsfile = self.getsettingsfile()
         writeoutconfigfile(self.settings,settingsfile)
 
-
     def readprefs(self):
-        # Check for GaussSum.ini in $HOME/GaussSum2.1 (XP)
-        #                        or $HOME/.gausssum2.1 (Linux)
 
-        appdata=os.getenv("HOME")
+        settingsfile = self.getsettingsfile()
         self.settings={}
 
-        if appdata==None: # Must not be a Windows machine!!
-            self.screen.write("Environmental variable HOME is not defined!\n\
-                               I cannot continue, as I need HOME to know\
-                               where to store GaussSum settings.\n")
-        else: # If everything is normal so far
-            settingsdir=os.path.join(appdata,"GaussSum2.1")
-            settingsfile=os.path.join(settingsdir,"GaussSum.ini")
+        if os.path.isfile(settingsfile): # Check for settings file
+            # Found it! - so read it in.
+            self.settings=readinconfigfile(settingsfile)
+            
+        else: # Initialise the settings file (and directory)
+            settingsdir = os.path.dirname(settingsfile)
+            if not os.path.isdir(settingsdir):
+                os.mkdir(settingsdir)
+            defaultgnuplot = os.path.join(installlocation,"gnuplot400","bin","wgnuplot.exe")
+            if not sys.platform == "win32":
+                defaultgnuplot = os.path.join(os.sep, "usr", "bin", "gnuplot")
 
-            if os.path.isfile(settingsfile): # Check for settings file
-                # Found it! - so read it in.
-                self.settings=readinconfigfile(settingsfile)
-                
-            else: # Initialise the settings file (and directory)
-                if not os.path.isdir(settingsdir):
-                    os.mkdir(settingsdir)
-                self.settings={
-##                               'global settings.filetype':'Gaussian',
-                               'global settings.gnuplot':os.path.join(installlocation,"gnuplot400","bin","wgnuplot.exe"),
-                               'find.text1':'SCF Done',
-                               'find.text2':'k 501%k502',
-                               'find.text3':'imaginary',
-                               'find.text4':'Framework',
-                               'ir_raman.start':'0',
-                               'ir_raman.end':'4000',
-                               'ir_raman.numpoints':'500',
-                               'ir_raman.fwhm':'3',
-                               'mo.start':'-20',
-                               'mo.end':'0',
-                               'mo.fwhm':'0.3',
-                               'uvvis.start':'300',
-                               'uvvis.end':'800',
-                               'uvvis.numpoints':'500',
-                               'uvvis.fwhm':'3000',
-                               'uvvis.sigma':'0.2',
+            self.settings={
+                           'global settings.gnuplot' : defaultgnuplot,
+                           'find.text1':'SCF Done',
+                           'find.text2':'k 501%k502',
+                           'find.text3':'imaginary',
+                           'find.text4':'Framework',
+                           'ir_raman.start':'0',
+                           'ir_raman.end':'4000',
+                           'ir_raman.numpoints':'500',
+                           'ir_raman.fwhm':'3',
+                           'mo.start':'-20',
+                           'mo.end':'0',
+                           'mo.fwhm':'0.3',
+                           'uvvis.start':'300',
+                           'uvvis.end':'800',
+                           'uvvis.numpoints':'500',
+                           'uvvis.fwhm':'3000',
+                           'uvvis.sigma':'0.2',
 ##                               'eddm.cubman':'C:\\Gaussian\\cubman.exe',
 ##                               'eddm.formchk':'C:\\Gaussian\\formchk.exe',
 ##                               'eddm.cubegen':'C:\\Gaussian\\cubegen.exe',
 ##                               'nmr.data':os.path.join(settingsdir,"nmrdata.txt")
-                               }
-                self.saveprefs() # Save the inital settings file
+                           }
+            self.saveprefs() # Save the inital settings file
 
  
     def webdocs(self):
-        webbrowser.open(os.path.join(installlocation,"Docs","index.html"))
+        webbrowser.open(os.path.join(installlocation,"..","Docs","index.html"))
     
 
